# -*- coding: utf-8 -*-
r"""
This module contains functions, which help to control the type conversion of command line parsing with argparse
"""
from pathlib import Path
from typing import Type
import argparse
import numpy as np


def pathtype(s: str) -> Path:
    r"""
    converts string to pathlib.Path object

    Args:
        s(str): string, which will be converted to Path

    Returns:
        converted Path-object
    """
    return Path(s)


def collect_as_tuple() -> Type[argparse.Action]:
    r"""
    Inner class CollectAsTuple inherits from argparse.Action and implements __call__. Control the correct reading of
    four range floats for defining rectangle range. Sets the attribute dest of CollectAsTuple to the tuple, which is
    accessible through argparse. For that purpose use the argparse action key.

    Returns:
        Object of CollectAsTuple, which is of Type argparse.Action
    """

    class CollectAsTuple(argparse.Action):
        def __call__(self, parser, namespace, values, options_string=None):
            if len(values) != 4:
                raise Exception('you have to provide four numbers as range for filtering to rectangle')
            if values[0] > values[1] or values[2] > values[3]:
                raise Exception('x2/y2 have to be greater than x1/y1')
            setattr(namespace, self.dest, (values[0], values[1], values[2], values[3]))

    return CollectAsTuple


def collect_as_nparray() -> Type[argparse.Action]:
    r"""
    Inner class CollectAsTuple inherits from argparse.Action and implements __call__. Control the correct reading of
    four range floats for defining rectangle range. Sets the attribute dest of CollectAsTuple to the numpy array,
    which is accessible through argparse. For that purpose use the argparse action key.

    Returns:
        Object of CollectAsTuple, which is of Type argparse.Action
    """

    class CollectAsNP(argparse.Action):
        def __call__(self, parser, namespace, values, options_string=None):
            setattr(namespace, self.dest, np.array([value for value in values]))

    return CollectAsNP


def boolean_string(s: str) -> bool:
    r"""
    boolean string conversion in argparse. If type=bool is provided every not empty string as argumented would be casted
    to True

    Args:
        s(str): string

    Returns:
        bool
    """
    if s not in ['False', 'True']:
        raise ValueError('Not a valid boolean string')
    return s == 'True'



# -*- coding: utf-8 -*-
from otterplot.configuration.iconfiguration import IConfiguration
from typing import Dict, Any


class CConfigFromYaml(IConfiguration):
    def __init__(self, yamlcontent: Dict[Any, Any], **kwargs) -> None:
        super().__init__()
        self.content = yamlcontent




# -*- coding: utf-8 -*-
from otterplot.configuration.iconfiguration import IConfiguration


class CConfigDefaultLatex(IConfiguration):
    def __init__(self,**kwargs) -> None:
        super().__init__()



# -*- coding: utf-8 -*-
r"""
prototype for an otterplotter-model.
"""
from otterplot.model.iotterplotter import IOtterPlotter
from otterplot.configuration.cconfigurationfactory import CConfigurationFactory
from pathlib import Path
from typing import Union, List, Tuple
from matplotlib.pyplot import figure, axis
from matplotlib.ticker import MultipleLocator


class COtterPlotter(IOtterPlotter):
    r"""
    prototype for plotter model
    """

    def __init__(self, plotmode: str, modebyyaml: Union[Path, None] = None, **kwargs) -> None:
        r"""
        initialize model

        Args:
            plotmode(str): Defines the settings.
            modebyyaml(Path): If the plot settings are provided by a single yaml file, this is the path to the file.
            Default value is None.
        """
        self._plotmode = plotmode
        self._config = CConfigurationFactory().create(plotmode=plotmode, yamlpath=modebyyaml, kwargs=kwargs)

    @property
    def plotmode(self) -> str:
        r"""
        Returns:
            The plotmode of the otterplotter model
        """
        return self._plotmode

    @property
    def figure(self) -> figure:
        r"""
        Returns:
            The figure of the plot
        """
        return self._config.figure

    @property
    def axes(self) -> List[axis]:
        r"""
        Returns:
            A list of all axis
        """
        return self._config.axes

    def adjustticklabels(self, ax_nr: Union[int, None] = None,
                         which: Tuple[bool, bool, bool, bool] = (True, False, True, False)) -> None:
        r"""
        Adjusts whether a specific axis (addressed by the ax_nr) should have labels on the left, right, bottom or top-
        side. Adjusts all axes if ax_nr is None.

        Args:
            ax_nr(int): integer addressing a specific axis. If None all axis are adjusted
            which(Tuple[bool, bool, bool, bool]): Which part of the axis is labeled (bottom, top, left, right). Default
            is (True, False, True, False).
        """
        if ax_nr is None:
            for ax in self._config.axes:
                ax.tick_params(axis='both',
                               which='major',
                               labelbottom=which[0],
                               labeltop=which[1],
                               labelleft=which[2],
                               labelright=which[3])
        else:
            if ax_nr > len(self._config.axes) - 1:
                raise ValueError('Not a valid axis number.')
            self._config.axes[ax_nr].tick_params(axis='both',
                                                 which='major',
                                                 labelbottom=which[0],
                                                 labeltop=which[1],
                                                 labelleft=which[2],
                                                 labelright=which[3])
#test
    def adjustlabels(self, ax_nr: Union[int, None] = None, xlabel: str = 'x', ylabel: str = 'y', xpos: str = 'bottom',
                     ypos: str = 'left') -> None:
        r"""
        Adjust the x- and y-labels and their position for a specific axis or for all axis if ax_nr is None.

        Args:
            ax_nr(Union[None, int]): the number of the specific axis. Default is None. Then all axis are addressed.
            xlabel(str): the text for the x label. The default is x.
            ylabel(str): the text for the y label. The default is y.
            xpos(str): position of the x label. Can be either bottom or top. Default is right.
            ypos(str): position of the y label. Can be either left or right. Default is left.
        """
        if xpos != 'bottom' and xpos != 'top':
            raise ValueError('Not a valid position for the x-axis.')
        if ypos != 'left' and ypos != 'right':
            raise ValueError('Not a valid position for the y-axis.')
        if ax_nr is None:
            for ax in self._config.axes:
                ax.set_xlabel(xlabel, fontsize=self._config.fontsize_labels)
                ax.set_ylabel(ylabel, fontsize=self._config.fontsize_labels)
                ax.xaxis.set_label_position(xpos)
                ax.yaxis.set_label_position(ypos)
        else:
            if ax_nr > len(self._config.axes) - 1:
                raise ValueError('Not a valid axis number.')
            self._config.axes[ax_nr].set_xlabel(xlabel, fontsize=self._config.fontsize_labels)
            self._config.axes[ax_nr].set_ylabel(ylabel, fontsize=self._config.fontsize_labels)
            self._config.axes[ax_nr].xaxis.set_label_position(xpos)
            self._config.axes[ax_nr].yaxis.set_label_position(ypos)

    def adjustlimits(self, ax_nr: Union[int, None] = None, xlims: Union[Tuple[float, float], None] = None,
                     ylims: Union[Tuple[float, float], None] = None) -> None:
        r"""
        Adjust the limits of all subplots.

        Args:
            ax_nr(Union[None, int]): the number of the specific axis. Default is None. Then all axis are addressed.
            xlims(Union[None, Tuple[float, float]]): the x limits as tuple. Default is None. Then the default limits
            from data are used, which matplotlib calculates.
            ylims(Union[None, Tuple[float, float]]): the y limits as tuple. Default is None. Then the default limits
            from data are used, which matplotlib calculates.
        """
        if ax_nr is None:
            for ax in self._config.axes:
                if xlims is not None:
                    ax.set_xlim(xlims)
                if ylims is not None:
                    ax.set_ylim(ylims)
        else:
            if ax_nr > len(self._config.axes) - 1:
                raise ValueError('Not a valid axis number.')
            if xlims is not None:
                self._config.axes[ax_nr].set_xlim(xlims)
            if ylims is not None:
                self._config.axes[ax_nr].set_ylim(ylims)

    def adjustticks(self, ax_nr: Union[int, None] = None, x_major: Union[float, None] = None,
                    x_minor: Union[float, None] = None, y_major: Union[float, None] = None,
                    y_minor: Union[float, None] = None):
        r"""
        Adjust the ticks of all subplots.

        Args:
           ax_nr(Union[None, int]): the number of the specific axis. Default is None. Then all axis are addressed.
           x_major(Union[None, float]): major ticks for x axis. Default is None. Then default matplotlib ticks are used.
           x_minor(Union[None, float]): minor ticks for x axis. Default is None. Then default matplotlib ticks are used.
           y_major(Union[None, float]): major ticks for y axis. Default is None. Then default matplotlib ticks are used.
           y_minor(Union[None, float]): minor ticks for y axis. Default is None. Then default matplotlib ticks are used.
        """
        if ax_nr is None:
            for ax in self._config.axes:
                if x_major is not None:
                    ax.xaxis.set_major_locator(MultipleLocator(x_major))
                if x_minor is not None:
                    ax.xaxis.set_minor_locator(MultipleLocator(x_minor))
                if y_major is not None:
                    ax.yaxis.set_major_locator(MultipleLocator(y_major))
                if y_minor is not None:
                    ax.yaxis.set_minor_locator(MultipleLocator(y_minor))
        else:
            if ax_nr > len(self._config.axes) - 1:
                raise ValueError('Not a valid axis number.')
            if x_major is not None:
                self._config.axes[ax_nr].xaxis.set_major_locator(MultipleLocator(x_major))
            if x_minor is not None:
                self._config.axes[ax_nr].xaxis.set_minor_locator(MultipleLocator(x_minor))
            if y_major is not None:
                self._config.axes[ax_nr].yaxis.set_major_locator(MultipleLocator(y_major))
            if y_minor is not None:
                self._config.axes[ax_nr].yaxis.set_minor_locator(MultipleLocator(y_minor))



# -*- coding: utf-8 -*-
from otterplot.process.parser.iparser import IParser
from otterplot.configuration.iconfiguration import IConfiguration
from otterplot.configuration.cconfigurationfactory import CConfigurationFactory
from pathlib import Path
from typing import Any, Dict, Union
import argparse


class CParser(IParser):
    r"""

    """

    def __init__(self, plotmode: str, show: bool, name: str, outformat: str, yaml: Path = None) -> None:
        super().__init__()
        self._plotmode = plotmode
        self._show = show
        self._name = name
        self._outformat = outformat
        self._yaml = yaml

    @property
    def outformat(self) -> str:
        return self._outformat

    @property
    def name(self) -> str:
        return self._name

    @property
    def show(self) -> bool:
        return self._show

    @property
    def plotmode(self) -> str:
        return self._plotmode

    @property
    def config(self) -> IConfiguration:
        r"""
        Returns:
             Configuration for 
        """
        return CConfigurationFactory.create(plotmode=self.plotmode, yamlpath=self._yaml)



    @staticmethod
    def parseinput() -> IParser:
        l_parser = argparse.ArgumentParser(description='Prototype Otterplot')
        l_parser.add_argument('--plotmode', type=str, help='mode for the plot',
                              choices=['paperPRB', 'defaultLatex', 'presentation', 'yaml'], required=True)
        l_parser.add_argument('--yaml', type=str, help='path to config.yaml', default='')
        l_parser.add_argument('--show', type=bool, default=False)
        l_parser.add_argument('--name', type=str, default='otter')
        l_parser.add_argument('--outformat', type=str, default='pdf')

        l_args = l_parser.parse_args()
        if l_args.plotmode == 'yaml' and not l_args.yaml:
            l_parser.error('If plotmode: yaml is selected, you need to provide a yaml file through --yaml file.')

        return CParser(plotmode=l_args.plotmode,
                       yaml=(lambda strpath: Path(strpath) if strpath else None)(l_args.yaml),
                       show=l_args.show,
                       name=l_args.name,
                       outformat=l_args.outformat
                       )



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from matplotlib.pyplot import figure, axis
from typing import List


class IConfiguration(ABC):
    r"""
    blueprint for the different configuration modes
    """

    @property
    @abstractmethod
    def figure(self) -> figure:
        r"""
        Returns:
            the figure
        """

    @property
    @abstractmethod
    def axes(self) -> List[axis]:
        r"""
        Returns:
            the list of axis
        """

    @property
    @abstractmethod
    def fontsize_labels(self) -> int:
        r"""
        Returns:
            The fontsize of the labels
        """



# -*- coding: utf-8 -*-
r"""
main module for testing the model api approach
"""
import matplotlib.pyplot as plt
from otterplot.model.cotterplotter import COtterPlotter
from otterplot.algorithm.converters import string_to_slice


def main() -> None:
    r"""
    main function
    """
    print(string_to_slice('0'))
    l_otterplotter = COtterPlotter(plotmode='paperPRB', extent='broad', figheight=4, grid=(6, 1),nonuniformax=[('0',':5'),('0','5')], axes3D=[0])
    print(len(l_otterplotter.axes))
    # new devel
    """l_otterplotter.adjustlimits(ax_nr=0, xlims=(0, 1), ylims=(2, 4))
    l_otterplotter.adjustticks(x_major=0.2, x_minor=0.05, y_major=0.5, y_minor=0.1)
    l_otterplotter.adjustticklabels(0, which=(False, True, True, False))
    l_otterplotter.adjustlabels(0, xlabel='xlabel', ylabel='ylabel',xpos='top',ypos='left')
    l_otterplotter.adjustticklabels(1, which=(False, True, False, True))
    l_otterplotter.adjustlabels(1,xlabel='xlabel', ylabel='ylabel',xpos='top',ypos='right')
    l_otterplotter.adjustticklabels(2, which=(True, False, True, False))
    l_otterplotter.adjustlabels(2,xlabel='xlabel', ylabel='ylabel',xpos='bottom',ypos='left')
    l_otterplotter.adjustticklabels(3, which=(True, False, False, True))
    l_otterplotter.adjustlabels(3,xlabel='xlabel', ylabel='ylabel',xpos='bottom',ypos='right')
    ax = l_otterplotter.axes[0]
    """
    # ax.plot([-5,5],[0,0],'k-')
    plt.show()
    plt.savefig('test.png')


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
r"""
This module contains the constants for the fontsizes prescribed by the physical review b format.
"""
# ======= options for figure ========
FIGUREWIDTH_NARROW = 3.38583
FIGUREWIDTH_BROAD = 3.38583 * 2
# ======= options for ticks =========
FONTSIZEMAJORTICKS = 8
TICKWIDTH_MAJOR = 1.5
TICKWIDTH_MINOR = 1.0
TICKLENGTH_MAJOR = 5.0
TICKLENGTH_MINOR = 3.0
TICKDIRECTION = 'in'
# ======= options fontsizes =======
FONTSIZEXYLABEL = 8


# -*- coding: utf-8 -*-
r"""
module contains factory pattern for configuration initialization depending on the plot mode
"""
import otterplot.configuration.constants.keywords as keys
from otterplot.configuration.cconfigurationdefaultlatex import CConfigDefaultLatex
from otterplot.configuration.cconfigurationfromyaml import CConfigFromYaml
from otterplot.configuration.cconfigurationpaperPRB import CConfigPaperPRB
from otterplot.configuration.cconfigurationpresentation import CConfigPresentation
from otterplot.configuration.iconfiguration import IConfiguration
from pathlib import Path
from typing import Dict, Any, Union

try:
    # Try to use the C implementation via LibYAML - if available.
    # Use SafeLoader and SafeDumper to avoid instantiation of arbitrary objects.
    from yaml import YAMLError, load as yamlload, CSafeLoader as SafeLoader  # type: ignore
except ImportError:
    # fallback
    from yaml import YAMLError, load as yamlload, SafeLoader  # type: ignore


class CConfigurationFactory:
    r"""
    Configuration Factory class.
    This class is used for creating a configuration object, which holds the information for the chosen plot type.
    """

    @classmethod
    def create(cls, plotmode: str, yamlpath: Union[Path, None] = None, kwargs: Union[Dict[str, Any], None] = None) -> IConfiguration:
        r"""
        Classmethod for creation of the configuration object for a plot type.

        Args:
            plotmode(str): the decider string for the factory pattern. The used keywords can be found in configuration/
            constants/keywords.py

            kwargs(Dict[str, Any]): any key, value pairs which the user can give to define options for the plot

            yamlpath(Union[Path, None]): Optional, if the plotmode is defined such as a yaml file is used as blueprint
            for the configuration object, this is the path to the specific yaml file.


        Returns:
            A configuration object inherited from CConfiguration
        """

        if plotmode == keys.PAPERPRB:
            config = CConfigPaperPRB(options=kwargs)
        elif plotmode == keys.DEFAULTLATEX:
            config = CConfigDefaultLatex()
        elif plotmode == keys.PRESENTATION:
            config = CConfigPresentation(options=kwargs)
        elif plotmode == keys.YAML:
            if yamlpath:
                config = CConfigFromYaml(yamlcontent=cls._load(yamlpath))
            else:
                raise Exception(f"If plotmode: {keys.YAML} is selected you must provide a path to the yaml"
                                f" to the factory.")
        else:
            raise ValueError(f"plotmode {plotmode} is not supported")

        return config

    @classmethod
    def _load(cls, yaml: Path) -> Union[Dict[Any, Any], None]:
        r"""
        Loads the yaml file into dictionary

        Args:
            yaml(Path): path to yaml

        Returns:
            Either a dict. or None

        Raises:
            IOError, YAMLError
        """
        if yaml:
            try:
                with open(yaml) as l_stream:
                    return yamlload(l_stream, Loader=SafeLoader)
            except(IOError, YAMLError):
                raise Exception("Error on reading yaml file")
        else:
            return None



# -*- coding: utf-8 -*-
r"""
blueprint for model classes. The usage should be in that way that the user initializes an otterplotter-object and
defines the configuration, which stores information about size, fontsize, ticklabelsizes, etc.
"""
from abc import ABC, abstractmethod


class IOtterPlotter(ABC):
    r"""
    blueprint for model classes
    """

    @property
    @abstractmethod
    def plotmode(self) -> str:
        r"""
        Returns:
            The plot mode of the otter plotter model
        """


# -*- coding: utf-8 -*-
r"""
This module contains the configuration class for the settings concerning figures in presentations.
"""
from typing import List, Union, Dict, Any
from matplotlib.pyplot import axis, figure
from matplotlib.gridspec import GridSpec
from otterplot.configuration.iconfiguration import IConfiguration
import otterplot.configuration.constants.constants_pres as PRES
import otterplot.configuration.constants.keywords as keys


class CConfigPresentation(IConfiguration):
    r"""
    Configuration class for the settings concerning figures in presentations.
    """

    def __init__(self, options: Union[Dict[str, Any], None]) -> None:
        r"""
        Initializes the configuration and therefore the settings for the plot

        Args: options: A dictionary with key value pairs. These are options the user is allowed to configure within
        the API. The allowed keys are defined in configuration.constants.keywords.py.
        """
        super().__init__()
        self._options = options
        self._setupbasics()
        self._applytickoptions()

    def _setupbasics(self) -> None:
        r"""
        Calls the initialization of the figure, the gridspec and the axes in the correct order.
        """
        self._figure = self._initialize_figure()
        self._gs = self._initialize_gridspec()
        self._axes = self._initialize_axes()

    def _initialize_axes(self) -> List[axis]:
        r"""
        Initializes the axis of the figure. The default is just one axis. Used for defining the axes-property, which is
        a list of axis. It is required to initialize the gridspec first.

        Returns:
            A list of axis.
        """
        l_cols, l_rows = self._gs.get_geometry()
        return [self._figure.add_subplot(self._gs[c, r]) for c in range(l_cols) for r in range(l_rows)]

    def _initialize_gridspec(self) -> GridSpec:
        r"""
        Initializes the GridSpec. The default subplot-grid has 1 column and 1 row.

        Returns:
            the gridspec
        """
        l_grid = self._options.get(keys.GRID, (1, 1))
        return GridSpec(ncols=l_grid[0], nrows=l_grid[1])

    def _initialize_figure(self) -> figure:
        r"""
        Initializes the figure. The default narrow width of a fig. in presentation format is defined in the belonging
        constants module. If no further options are provided the figure will be of quadratic with the size defined in
        the narrow type. If the option keys.FIGUREHEIGHT is given the figure will be
        of (PRES.FIGUREWIDTH_NARROW,fig_height). If keys.EXTENTYPE is equal to keys.BROAD the output figure will have
        size (PRES.FIGUREWIDTH_BROAD, fig_height).

        Returns:
            the figure
        """
        l_fig = figure(figsize=(PRES.FIGUREWIDTH_NARROW, PRES.FIGUREWIDTH_NARROW))
        if self._options is None:
            return l_fig
        # the height can be adjusted if not specified the height so set to the width of a narrow version
        l_fig_height = self._options.get(keys.FIGUREHEIGHT, PRES.FIGUREWIDTH_NARROW)
        l_extend = self._options.get(keys.EXTENTTYPE, keys.NARROW)
        if l_extend == keys.NARROW:
            return figure(figsize=(PRES.FIGUREWIDTH_NARROW, l_fig_height))
        elif l_extend == keys.TINY:
            return figure(figsize=(PRES.FIGUREWIDTH_TINY, l_fig_height))
        elif l_extend == keys.BROAD:
            return figure(figsize=(PRES.FIGUREWIDTH_BROAD, l_fig_height))
        else:
            raise ValueError('not a valid key for extent')

    def _applytickoptions(self) -> None:
        r"""
        Applies options to the axis, including tick labelsize, tick length and width, the direction of the ticks.
        Per Default all axis are enabled
        """
        for ax in self._axes:
            ax.tick_params(axis='both',
                           which='major',
                           labelsize=PRES.FONTSIZEMAJORTICKS,
                           length=PRES.TICKLENGTH_MAJOR,
                           width=PRES.TICKWIDTH_MAJOR,
                           direction=PRES.TICKDIRECTION,
                           bottom=True,
                           top=True,
                           left=True,
                           right=True,
                           labelbottom=True,
                           labeltop=False,
                           labelleft=True,
                           labelright=False)

            ax.tick_params(axis='both',
                           which='minor',
                           direction=PRES.TICKDIRECTION,
                           length=PRES.TICKLENGTH_MINOR,
                           width=PRES.TICKWIDTH_MINOR,
                           bottom=True,
                           top=True,
                           left=True,
                           right=True)

    @property
    def fontsize_labels(self) -> int:
        r"""
        Returns:
            The fontsize of the labels
        """
        return PRES.FONTSIZEXYLABEL

    @property
    def axes(self) -> List[axis]:
        r"""
        Returns:
            A list of axis
        """
        return self._axes

    @property
    def figure(self) -> figure:
        r"""
        Returns:
             The figure for the plot.
        """
        return self._figure

    def __str__(self) -> str:
        r"""
        Returns:
             The string representation of the configuration object
        """
        return f"plotmode {keys.PRESENTATION}"



# -*- coding: utf-8 -*-
r"""
This module contains the keywords for the factory pattern classes used in this project.
"""
# =======================Creating Plotmode===========================
PAPERPRB = 'paperPRB'
DEFAULTLATEX = 'defaultLatex'
PRESENTATION = 'presentation'
YAML = 'yaml'
# =======================Specific Options for Plotmode===============

# Options for defining the Figure. The inputs are designed in a way that only figures within the allowed rules depending
# on the format can be created.
# -------------------------------------------------------------------
EXTENTTYPE = 'extent'
NARROW = 'narrow'
BROAD = 'broad'
TINY = 'tiny'
# figure height in inches, input type has to be float
FIGUREHEIGHT = 'figheight'
# -------------------------------------------------------------------
# Gridspec defines number of subplots with number of rows and number of columns data is provided as Tuple
GRID = 'grid'
# -------------------------------------------------------------------
# List of specific axes which shall be in 3D projection mode
AXES3D = 'axes3D'
# In default mode a grid of uniform axes will be created. Sometimes one wants to have one axis bigger than another.
# E.g. a plot and a colorbar
NONUNIFORMAX = 'nonuniformax'



r"""
Module which visualizes the spin lattice with pyvista
"""
import pyvista as pv
import pandas as pd
import numpy as np
from pathlib import Path


class CVisualizeStructPyVista:
    r"""
    Class for visualizing 1D domain wall
    """

    def __init__(self, file: Path, tiplength: float = 1.0, tipradius: float = 0.35, arrowscale: float = 0.7) -> None:
        r"""
        Initializes the the visualization

        Args:
            file(Path): STM file
            tiplength(float): geometry of arrow: tiplength
            tipradius(float): geometry of arrow: tipradius
            arrowscale(float): geometry of arrow: arrowscale
        """
        self.file = file
        df = pd.read_csv(self.file, sep=r'\s+', usecols=[0, 1, 2, 3, 4, 5], names=['x', 'y', 'z', 'sx', 'sy', 'sz'])
        self.x = df['x'].to_numpy()
        self.y = df['y'].to_numpy()
        self.z = df['z'].to_numpy()
        self.sx = df['sx'].to_numpy()
        self.sy = df['sy'].to_numpy()
        self.sz = df['sz'].to_numpy()
        self.r = np.column_stack((self.x, self.y, self.z))
        self.s = np.column_stack((self.sx, self.sy, self.sz))

        self.tiplength = tiplength
        self.tipradius = tipradius
        self.arrowscale = arrowscale

    def __call__(self) -> pv.Plotter:
        r"""
        Calls the visualization.

        Returns:
            the pyvista plot object.
        """
        geom = pv.Arrow(start=np.array([-self.arrowscale / 2.0, 0, 0]), tip_length=self.tiplength,
                        tip_radius=self.tipradius, scale=self.arrowscale)
        self.p = pv.Plotter(off_screen=False, lighting='three lights')
        pv.set_plot_theme("ParaView")
        pv.rcParams['transparent_background'] = True
        self.p.set_background('white')
        structure_pd = pv.PolyData(self.r)
        structure_pd.vectors = self.s
        structure_pd['oop'] = self.sz
        structure_pd_glyphs = structure_pd.glyph(orient=True, scale=True, geom=geom)

        self.p.add_mesh(structure_pd_glyphs, scalars='oop', show_scalar_bar=False)

        def cam() -> None:
            print('Camera postion', self.p.camera_position)

        self.p.add_key_event('c', cam)
        return self.p

    def set_camera(self, cam: np.array) -> None:
        r"""
        Sets the camera of the plotter.
        """
        self.p.camera = cam


    def screenshot(self, name_screenshot: str = 'STM') -> None:
        r"""
        Takes a screenshot
        """
        self.p.ren_win.OffScreenRenderingOn()
        self.p.window_size=[4000,4000]
        self.p.screenshot(f'{name_screenshot}.png')
        self.p.ren_win.OffScreenRenderingOff()



# -*- coding: utf-8 -*-
r"""
This module contains the configuration class for the settings concerning figures in physical review B papers.
"""
from matplotlib.pyplot import figure, axis
import otterplot.configuration.constants.keywords as keys
from typing import Union, Dict, Any, List
from otterplot.configuration.iconfiguration import IConfiguration
from otterplot.algorithm.converters import string_to_slice
from matplotlib.gridspec import GridSpec
import otterplot.configuration.constants.constants_prb as PRB
from mpl_toolkits.mplot3d import Axes3D


class CConfigPaperPRB(IConfiguration):
    r"""
    Configuration class for the settings concerning figures in physical review b papers.
    """

    def __init__(self, options: Union[Dict[str, Any], None]) -> None:
        r"""
        Initializes the configuration and therefore the settings for the plot

        Args: options: A dictionary with key value pairs. These are options the user is allowed to configure within
        the API. The allowed keys are defined in configuration.constants.keywords.py.
        """
        super().__init__()
        self._options = options
        self._setupbasics()
        self._applytickoptions()

    def _setupbasics(self) -> None:
        r"""
        Calls the initialization of the figure, the gridspec and the axes in the correct order.
        """
        self._figure = self._initialize_figure()
        self._gs = self._initialize_gridspec()
        self._axes = self._initialize_axes()

    def _initialize_gridspec(self) -> GridSpec:
        r"""
        Initializes the GridSpec. The default subplot-grid has 1 column and 1 row.

        Returns:
            the gridspec
        """
        l_grid = self._options.get(keys.GRID, (1, 1))
        return GridSpec(ncols=l_grid[0], nrows=l_grid[1])

    def _initialize_axes(self) -> List[axis]:
        r"""
        Initializes the axis of the figure. The default is just one axis. Used for defining the axes-property, which is
        a list of axis. It is required to initialize the gridspec first. This method uses a lot of options:

        - If the parameter nonuniformax is not provided as matrix of axes of the same size is created for the grid
        defined. The parameter axes3D defines the index of the axes those shall be in 3d -projection mode.

        - If the parameter nonuniformax is provided (A list of tuples T for each tuple T =(stringslice,stringslice) repr.
        a string which can be converted into a slice for the row or column, respectively. If the parameter axes3D is
        provided some axis are presented in 3d projection mode

        Returns:
            A list of axis.
        """
        l_cols, l_rows = self._gs.get_geometry()
        axes3D = self._options.get(keys.AXES3D, None)
        nonuniformax = self._options.get(keys.NONUNIFORMAX, None)
        if nonuniformax is None:
            if axes3D is None:
                return [self._figure.add_subplot(self._gs[c, r]) for c in range(l_cols) for r in range(l_rows)]
            else:
                if isinstance(axes3D, list):
                    return [self._figure.add_subplot(self._gs[c, r],
                                                     projection='3d') if r + c in axes3D else self._figure.add_subplot(
                        self._gs[c, r]) for c in range(l_cols) for r in range(l_rows)]
                else:
                    raise ValueError("the 3d axis have to be provided as list of indices")
        else:
            if isinstance(nonuniformax, list):
                if axes3D is None:
                    return [self._figure.add_subplot(
                        self._gs[string_to_slice(slicetuple[0]), string_to_slice(slicetuple[1])]) for slicetuple in
                            nonuniformax]
                else:
                    l_axs = []
                    for (idx, slicetuple) in enumerate(nonuniformax):
                        if idx in axes3D:
                            print(slicetuple[0])
                            print(slicetuple[1])
                            l_axs.append(self._figure.add_subplot(
                                self._gs[string_to_slice(slicetuple[0]), string_to_slice(slicetuple[1])],
                                projection='3d'))
                        else:
                            l_axs.append(self._figure.add_subplot(
                                self._gs[string_to_slice(slicetuple[0]), string_to_slice(slicetuple[1])]))
                    return l_axs
            else:
                raise ValueError("the nonuniformax parameter has to be a list of tuples. Each tuple has to be"
                                 "a pair of strings representing a slice")

    def _initialize_figure(self) -> figure:
        r"""
        Initializes the figure. The size for a column in physical review B is 3.38583 inch. If no further options are
        provided the figure will be of (3.38583,3.38583). If the option keys.FIGUREHEIGHT is given the figure will be
        of (3.38583,fig_height). If keys.EXTENTYPE is equal to keys.BROAD the output figure will have size
        (3.38583 * 2, fig_height) as a two columnar figure in PRB.
        """
        l_fig = figure(figsize=(PRB.FIGUREWIDTH_NARROW, PRB.FIGUREWIDTH_NARROW))
        if self._options is None:
            return l_fig
        # the height can be adjusted
        l_fig_height = self._options.get(keys.FIGUREHEIGHT, PRB.FIGUREWIDTH_NARROW)
        l_extend = self._options.get(keys.EXTENTTYPE, keys.NARROW)
        if l_extend == keys.NARROW:
            return figure(figsize=(PRB.FIGUREWIDTH_NARROW, l_fig_height))
        elif l_extend == keys.BROAD:
            return figure(figsize=(PRB.FIGUREWIDTH_BROAD, l_fig_height))
        else:
            raise ValueError('not a valid key for extent')

    def _applytickoptions(self) -> None:
        r"""
        Applies options to the axis, including tick labelsize, tick length and width, the direction of the ticks.
        Per Default all axis are enabled
        """
        for ax in self._axes:
            ax.tick_params(axis='both',
                           which='major',
                           labelsize=PRB.FONTSIZEMAJORTICKS,
                           length=PRB.TICKLENGTH_MAJOR,
                           width=PRB.TICKWIDTH_MAJOR,
                           direction=PRB.TICKDIRECTION,
                           bottom=True,
                           top=True,
                           left=True,
                           right=True,
                           labelbottom=True,
                           labeltop=False,
                           labelleft=True,
                           labelright=False)

            ax.tick_params(axis='both',
                           which='minor',
                           direction=PRB.TICKDIRECTION,
                           length=PRB.TICKLENGTH_MINOR,
                           width=PRB.TICKWIDTH_MINOR,
                           bottom=True,
                           top=True,
                           left=True,
                           right=True)

    @property
    def fontsize_labels(self) -> int:
        r"""
        Returns:
            The fontsize of the labels
        """
        return PRB.FONTSIZEXYLABEL

    @property
    def axes(self) -> List[axis]:
        r"""
        Returns:
            A list of axis
        """
        return self._axes

    @property
    def figure(self) -> figure:
        r"""
        Returns:
             The figure for the plot.
        """
        return self._figure

    def __str__(self) -> str:
        r"""
        Returns:
             The string representation of the configuration object
        """
        return f"plotmode {keys.PAPERPRB}"



# -*- coding: utf-8 -*-
from otterplot.process.parser.cparser import CParser
from otterplot.process.parser.iparser import IParser


def main(parser: IParser):
    config = parser.config
    print(parser.config.content)


if __name__ == "__main__":
    main(CParser.parseinput())



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from otterplot.configuration.iconfiguration import IConfiguration

class IParser(ABC):
    r"""

    """
    @property
    @abstractmethod
    def plotmode(self) -> str:
        r"""
        Gets the plotmode property for implementations of IParser.
        :return: (str)
        """

    @property
    @abstractmethod
    def show(self) -> bool:
        r"""
        Gets the show property for implementations of IParser.
        :return: (bool)
        """

    @property
    @abstractmethod
    def name(self) -> str:
        r"""
        Gets the name property for implementations of IParser.
        :return: (str)
        """

    @property
    @abstractmethod
    def outformat(self) -> str:
        r"""
        Gets the outformat property for implementations of IParser.
        :return: (str)
        """


    @property
    @abstractmethod
    def config(self) -> IConfiguration:
        r"""
        Gets the configuration property for implementations of IParser.
        :return: (CConfiguration) instance of specific configuration object representing the plotmode.
        """



r"""
This module contains a few converting functions as utilities for the library.
"""
from typing import Union


def string_to_slice(slicestr: str) -> Union[slice, int]:
    r"""
    Converts a string representation of a slice in an actual slice. For example useful for the definitions of GridSpec

    Args:
        slicestr(str): a string representing a slice. E.g. 1:, :-1,2:3,4

    Returns:
        the slice representation. E.g: slice(1,None),slice(None,-1),slice(2,3),slice(4)
    """
    if slicestr == ':':
        return slice(None, None)
    else:
        if ':' in slicestr:
            parts = slicestr.split(':')
            for (idx, p) in enumerate(parts):
                if p == '':
                    parts[idx] = None
                else:
                    parts[idx] = int(p)
            if len(parts) == 2:
                return slice(parts[0], parts[1])
            elif len(parts) == 3:
                return slice(parts[0], parts[1], parts[2])
            else:
                raise ValueError('not a valid slice representation')
        else:
            try:
                return int(slicestr)
            except ValueError:
                raise ValueError('Cannot convert input string into single integer while converting slice')
